@@ -1,12 +1,12 @@
-from . import extras
-from ._cli import cli, generate_parser
+from . import conf, extras
+from ._cli import cli
 from ._fields import MISSING_PUBLIC as MISSING
 from ._instantiators import UnsupportedTypeAnnotationError
 
 __all__ = [
+    "conf",
     "extras",
     "cli",
-    "generate_parser",
     "MISSING",
     "UnsupportedTypeAnnotationError",
 ]
