@@ -6,7 +6,6 @@ import dataclasses
 import itertools
 from typing import Any, Callable, Dict, List, Optional, Set, Type, TypeVar, Union, cast
 
-import termcolor
 from typing_extensions import get_args, get_origin
 
 from . import (
@@ -18,6 +17,7 @@ from . import (
     _resolver,
     _strings,
 )
+from .conf import _markers, _subcommands
 
 T = TypeVar("T")
 
@@ -26,6 +26,7 @@ T = TypeVar("T")
 class ParserSpecification:
     """Each parser contains a list of arguments and optionally some subparsers."""
 
+    f: Callable
     description: str
     args: List[_arguments.ArgumentDefinition]
     helptext_from_nested_class_field_name: Dict[str, Optional[str]]
@@ -42,7 +43,6 @@ class ParserSpecification:
             T, _fields.PropagatingMissingType, _fields.NonpropagatingMissingType
         ],
         prefix: str,
-        avoid_subparsers: bool,
     ) -> ParserSpecification:
         """Create a parser definition from a callable."""
 
@@ -78,74 +78,88 @@ class ParserSpecification:
         for field in field_list:
             field = dataclasses.replace(
                 field,
+                # Resolve generic types.
                 typ=_resolver.type_from_typevar_constraints(
-                    type_from_typevar.get(  # type: ignore
-                        field.typ,
+                    _resolver.apply_type_from_typevar(
                         field.typ,
+                        type_from_typevar,
                     )
                 ),
             )
+
             if isinstance(field.typ, TypeVar):
                 raise _instantiators.UnsupportedTypeAnnotationError(
                     f"Field {field.name} has an unbound TypeVar: {field.typ}."
                 )
 
-            # (1) Handle Unions over callables; these result in subparsers.
-            subparsers_attempt = SubparsersSpecification.from_field(
-                field,
-                type_from_typevar=type_from_typevar,
-                parent_classes=parent_classes,
-                prefix=_strings.make_field_name([prefix, field.name]),
-                avoid_subparsers=avoid_subparsers,
-            )
-            if subparsers_attempt is not None:
-                if (
-                    not avoid_subparsers
-                    # Required subparsers => must create a subparser.
-                    or subparsers_attempt.required
-                    # If the output can be None, the only way to specify whether this is
-                    # or isn't None is with a subparser.
-                    or subparsers_attempt.can_be_none
-                ):
-                    subparsers_from_name[subparsers_attempt.name] = subparsers_attempt
-                    continue
-                else:
-                    field = dataclasses.replace(field, typ=type(field.default))
-                    assert _fields.is_nested_type(field.typ, field.default)
-
-            # (2) Handle nested callables.
-            if _fields.is_nested_type(field.typ, field.default):
-                field = dataclasses.replace(
-                    field, typ=_resolver.narrow_type(field.typ, field.default)
-                )
-                nested_parser = ParserSpecification.from_callable(
-                    field.typ,
-                    description=None,
+            if _markers.FIXED not in field.markers:
+                # (1) Handle Unions over callables; these result in subparsers.
+                subparsers_attempt = SubparsersSpecification.from_field(
+                    field,
+                    type_from_typevar=type_from_typevar,
                     parent_classes=parent_classes,
-                    parent_type_from_typevar=type_from_typevar,
-                    default_instance=field.default,
                     prefix=_strings.make_field_name([prefix, field.name]),
-                    avoid_subparsers=avoid_subparsers,
                 )
-                args.extend(nested_parser.args)
+                if subparsers_attempt is not None:
+                    if (
+                        not subparsers_attempt.required
+                        and _markers.AVOID_SUBCOMMANDS in field.markers
+                    ):
+                        # Don't make a subparser.
+                        field = dataclasses.replace(field, typ=type(field.default))
+                    else:
+                        subparsers_from_name[
+                            _strings.make_field_name([prefix, subparsers_attempt.name])
+                        ] = subparsers_attempt
+                        continue
 
-                for k, v in nested_parser.helptext_from_nested_class_field_name.items():
-                    helptext_from_nested_class_field_name[
-                        _strings.make_field_name([field.name, k])
-                    ] = v
+                # (2) Handle nested callables.
+                if _fields.is_nested_type(field.typ, field.default):
+                    field = dataclasses.replace(
+                        field,
+                        typ=_resolver.narrow_type(
+                            field.typ,
+                            field.default,
+                        ),
+                    )
+                    nested_parser = ParserSpecification.from_callable(
+                        field.typ,
+                        description=None,
+                        parent_classes=parent_classes,
+                        parent_type_from_typevar=type_from_typevar,
+                        default_instance=field.default,
+                        prefix=_strings.make_field_name([prefix, field.name]),
+                    )
+                    args.extend(nested_parser.args)
 
-                if field.helptext is not None:
-                    helptext_from_nested_class_field_name[field.name] = field.helptext
-                else:
-                    helptext_from_nested_class_field_name[
-                        field.name
-                    ] = _docstrings.get_callable_description(field.typ)
-                continue
+                    # Include nested subparsers.
+                    subparsers_from_name.update(nested_parser.subparsers_from_name)
 
-            # (3) Handle primitive types. These produce a single argument!
+                    # Include nested strings.
+                    for (
+                        k,
+                        v,
+                    ) in nested_parser.helptext_from_nested_class_field_name.items():
+                        helptext_from_nested_class_field_name[
+                            _strings.make_field_name([field.name, k])
+                        ] = v
+
+                    if field.helptext is not None:
+                        helptext_from_nested_class_field_name[
+                            _strings.make_field_name([field.name])
+                        ] = field.helptext
+                    else:
+                        helptext_from_nested_class_field_name[
+                            _strings.make_field_name([field.name])
+                        ] = _docstrings.get_callable_description(field.typ)
+                    continue
+
+            # (3) Handle primitive or fixed types. These produce a single argument!
             args.append(
                 _arguments.ArgumentDefinition(
-                    prefix=prefix, field=field, type_from_typevar=type_from_typevar
+                    prefix=prefix,
+                    field=field,
+                    type_from_typevar=type_from_typevar,
                 )
             )
 
@@ -159,6 +173,7 @@ class ParserSpecification:
             )
 
         return ParserSpecification(
+            f=f,
             description=description
             if description is not None
             else _docstrings.get_callable_description(f),
@@ -176,9 +191,7 @@ class ParserSpecification:
 
         # Make argument groups.
         def format_group_name(nested_field_name: str) -> str:
-            return termcolor.colored(
-                (nested_field_name + " arguments").strip(), attrs=["bold"]
-            )
+            return (nested_field_name + " arguments").strip()
 
         group_from_prefix: Dict[str, argparse._ArgumentGroup] = {
             "": parser._action_groups[1],
@@ -186,25 +199,35 @@ class ParserSpecification:
 
         # Break some API boundaries to rename the optional group.
         parser._action_groups[1].title = format_group_name("")
-        positional_group = parser.add_argument_group(
-            termcolor.colored("positional arguments", attrs=["bold"])
-        )
+        positional_group = parser.add_argument_group("positional arguments")
         parser._action_groups = parser._action_groups[::-1]
 
+        # Add each argument group. Note that groups with only suppressed arguments won't
+        # be added.
+        for arg in self.args:
+            if (
+                arg.lowered.help is not argparse.SUPPRESS
+                and arg.prefix not in group_from_prefix
+            ):
+                description = self.helptext_from_nested_class_field_name.get(arg.prefix)
+                group_from_prefix[arg.prefix] = parser.add_argument_group(
+                    format_group_name(arg.prefix),
+                    description=description,
+                )
+
         # Add each argument.
         for arg in self.args:
-            if arg.field.positional:
+            if arg.field.is_positional():
                 arg.add_argument(positional_group)
                 continue
 
-            if arg.prefix not in group_from_prefix:
-                group_from_prefix[arg.prefix] = parser.add_argument_group(
-                    format_group_name(arg.prefix),
-                    description=self.helptext_from_nested_class_field_name.get(
-                        arg.prefix
-                    ),
-                )
-            arg.add_argument(group_from_prefix[arg.prefix])
+            if arg.prefix in group_from_prefix:
+                arg.add_argument(group_from_prefix[arg.prefix])
+            else:
+                # Suppressed argument: still need to add them, but they won't show up in
+                # the helptext so it doesn't matter which group.
+                assert arg.lowered.help is argparse.SUPPRESS
+                arg.add_argument(group_from_prefix[""])
 
         # Create subparser tree.
         if len(self.subparsers_from_name) > 0:
@@ -233,14 +256,17 @@ class SubparsersSpecification:
         type_from_typevar: Dict[TypeVar, Type],
         parent_classes: Set[Type],
         prefix: str,
-        avoid_subparsers: bool,
     ) -> Optional[SubparsersSpecification]:
         # Union of classes should create subparsers.
-        if get_origin(field.typ) is not Union:
+        typ = _resolver.unwrap_annotated(field.typ)[0]
+        if get_origin(typ) is not Union:
             return None
 
         # We don't use sets here to retain order of subcommands.
-        options = [type_from_typevar.get(typ, typ) for typ in get_args(field.typ)]
+        options = [
+            _resolver.apply_type_from_typevar(typ, type_from_typevar)
+            for typ in get_args(typ)
+        ]
         options_no_none = [o for o in options if o != type(None)]  # noqa
         if not all(
             [
@@ -253,19 +279,45 @@ class SubparsersSpecification:
         # Add subparser for each option.
         parser_from_name: Dict[str, ParserSpecification] = {}
         for option in options_no_none:
-            subparser_name = _strings.subparser_name_from_type(prefix, option)
-            parser_from_name[subparser_name] = ParserSpecification.from_callable(
+            name = _strings.subparser_name_from_type(prefix, option)
+            option, found_subcommand_configs = _resolver.unwrap_annotated(
+                option, _subcommands._SubcommandConfiguration
+            )
+            if len(found_subcommand_configs) == 0:
+                # Make a dummy subcommand config.
+                found_subcommand_configs = (
+                    _subcommands._SubcommandConfiguration(
+                        "unused",
+                        description=None,
+                        default=(
+                            field.default
+                            if type(field.default)
+                            is _resolver.unwrap_origin_strip_extras(option)
+                            else _fields.MISSING_NONPROP
+                        ),
+                        prefix_name=True,
+                    ),
+                )
+
+            subparser = ParserSpecification.from_callable(
                 option,
-                description=None,
+                description=found_subcommand_configs[0].description,
                 parent_classes=parent_classes,
                 parent_type_from_typevar=type_from_typevar,
-                default_instance=field.default
-                if type(field.default) == _resolver.unwrap_origin(option)  # type: ignore
-                else _fields.MISSING_NONPROP,
+                default_instance=found_subcommand_configs[0].default,
                 prefix=prefix,
-                avoid_subparsers=avoid_subparsers,
             )
 
+            # Apply prefix to helptext in nested classes in subparsers.
+            subparser = dataclasses.replace(
+                subparser,
+                helptext_from_nested_class_field_name={
+                    _strings.make_field_name([prefix, k]): v
+                    for k, v in subparser.helptext_from_nested_class_field_name.items()
+                },
+            )
+            parser_from_name[name] = subparser
+
         # Optional if: type hint is Optional[], or a default instance is provided.
         required = True
         if field.default not in _fields.MISSING_SINGLETONS:
@@ -273,6 +325,7 @@ class SubparsersSpecification:
 
         # If there are any required arguments in the default subparser, we should mark
         # the subparser group as a whole as required.
+        default_name = None
         if (
             field.default is not None
             and field.default not in _fields.MISSING_SINGLETONS
@@ -284,9 +337,28 @@ class SubparsersSpecification:
                     "Default values for generic subparsers are not supported."
                 )
 
-            default_parser = parser_from_name[
-                _strings.subparser_name_from_type(prefix, type(field.default))
-            ]
+            default_name = _strings.subparser_name_from_type(
+                prefix, type(field.default)
+            )
+            if default_name not in parser_from_name:
+                # If we can't find the subparser by name, search by type. This is needed
+                # when the user renames their subcommands. (eg via dcargs.subcommand)
+                #
+                # TODO: this will display some weird behaviors if multiple subcommands
+                # have the same type.
+                default_name = None
+
+                for name, parser in parser_from_name.items():
+                    if type(field.default) is _resolver.unwrap_origin_strip_extras(
+                        parser.f
+                    ):
+                        default_name = name
+                        break
+                assert default_name is not None, (
+                    f"Default with type {type(field.default)} was passed in, but no"
+                    " matching subparser."
+                )
+            default_parser = parser_from_name[default_name]
             if any(map(lambda arg: arg.lowered.required, default_parser.args)):
                 required = True
             if any(
@@ -302,10 +374,7 @@ class SubparsersSpecification:
         if field.helptext is not None:
             description_parts.append(field.helptext)
         if not required and field.default not in _fields.MISSING_SINGLETONS:
-            default = field.default
-            if default is not None:
-                default = _strings.subparser_name_from_type(prefix, type(default))
-            description_parts.append(f" (default: {default})")
+            description_parts.append(f" (default: {default_name})")
         description = (
             # We use `None` instead of an empty string to prevent a line break from
             # being created where the description would be.
@@ -355,29 +424,26 @@ class SubparsersSpecification:
         for p in prev_subparser_tree_nodes:
             # Add subparsers to every node in previous level of the tree.
             argparse_subparsers = p.add_subparsers(
-                dest=_strings.make_field_name(
-                    [
-                        parent_parser.prefix,
-                        _strings.make_subparser_dest(name=self.name),
-                    ]
-                ),
+                dest=_strings.make_subparser_dest(self.prefix),
                 description=self.description,
                 required=self.required,
-                title=termcolor.colored(title, attrs=["bold"]),
+                title=title,
                 metavar=metavar,
             )
 
             if self.can_be_none:
                 subparser = argparse_subparsers.add_parser(
                     name=_strings.subparser_name_from_type(self.prefix, None),
-                    formatter_class=_argparse_formatter.ArgparseHelpFormatter,
+                    formatter_class=_argparse_formatter.DcargsArgparseHelpFormatter,
+                    help="",
                 )
                 subparser_tree_nodes.append(subparser)
 
             for name, subparser_def in self.parser_from_name.items():
                 subparser = argparse_subparsers.add_parser(
                     name,
-                    formatter_class=_argparse_formatter.ArgparseHelpFormatter,
+                    formatter_class=_argparse_formatter.DcargsArgparseHelpFormatter,
+                    help=subparser_def.description,
                 )
                 subparser_def.apply(subparser)
 
